//
// Copyright (c) ZeroC, Inc. All rights reserved.
//

#ifndef ICE_BATCH_REQUEST_QUEUE_F_H
#define ICE_BATCH_REQUEST_QUEUE_F_H

#include <IceUtil/Shared.h>
#include <Ice/Handle.h>

namespace IceInternal
{

class BatchRequestQueue;
ICE_API IceUtil::Shared* upCast(BatchRequestQueue*);
typedef IceInternal::Handle<BatchRequestQueue> BatchRequestQueuePtr;

}

#endif
