//
// Copyright (c) ZeroC, Inc. All rights reserved.
//

#ifndef ICE_VALUE_F_H
#define ICE_VALUE_F_H

#ifdef ICE_CPP11_MAPPING
#include <Ice/Config.h>

namespace Ice
{

class Value;
/// \cond INTERNAL
using ValuePtr = ::std::shared_ptr<Value>;
/// \endcond

}
#endif
#endif
